from supybot import conf, registry
try:
    from supybot.i18n import PluginInternationalization
    _ = PluginInternationalization('Sourcehut')
except:
    _ = lambda x: x


def configure(advanced):
    from supybot.questions import expect, anything, something, yn
    conf.registerPlugin('Sourcehut', True)


Sourcehut = conf.registerPlugin('Sourcehut')



import email.header
import email.utils
import json
import mailbox
from urllib.parse import quote
from urllib.request import urlopen
import re
import traceback

from supybot import callbacks, httpserver, ircmsgs, world
from supybot.ircutils import bold, italic, mircColor, underline


class Sourcehut(callbacks.Plugin):
    """
    Supybot plugin to receive Sourcehut webhooks
    """

    def __init__(self, irc):
        super().__init__(irc)
        httpserver.hook("sourcehut", SourcehutServerCallback(self))

    def die(self):
        httpserver.unhook("sourcehut")
        super().die()

    def announce(self, channel, message):
        libera = world.getIrc("libera")
        if libera is None:
            print("error: no irc libera")
            return
        if channel not in libera.state.channels:
            print(f"error: not in {channel} channel")
            return
        libera.sendMsg(ircmsgs.notice(channel, message))


def decode_header(header: str) -> str:
    if not header:
        return ""
    text = ""
    for chunk, encoding in email.header.decode_header(header):
        if isinstance(chunk, bytes):
            chunk = chunk.decode(encoding or "us-ascii")
        text += chunk
    return text


class SourcehutServerCallback(httpserver.SupyHTTPServerCallback):
    name = "Sourcehut"
    defaultResponse = "Bad request\n"

    def __init__(self, plugin: Sourcehut):
        super().__init__()
        self.plugin = plugin

    SUBJECT = "[PATCH {prefix} v{version}] {subject}"
    URL = "https://lists.sr.ht/{list[owner][canonicalName]}/{list[name]}"
    CHANS = {
        "#public-inbox": "##rjarry",
        "#aerc-devel": "#aerc",
    }

    def announce_patch(self, patchset):
        subject = self.SUBJECT.format(**patchset)
        url = self.URL.format(**patchset)
        if not url.startswith("https://lists.sr.ht/~rjarry/"):
            raise ValueError("unknown list")
        url += "/patches/{id}".format(**patchset)
        channel = f"#{patchset['list']['name']}"
        channel = self.CHANS.get(channel, channel)
        try:
            submitter = patchset["submitter"]["canonicalName"]
        except KeyError:
            try:
                submitter = patchset["submitter"]["name"]
            except KeyError:
                submitter = patchset["submitter"]["address"]
        msg = f"{mircColor('received', 'light gray')} {bold(subject)}"
        msg += f" from {italic(submitter)}: {underline(url)}"
        self.plugin.announce(channel, msg)

    def announce_apply(self, mail):
        channel = f"#{mail['list']['name']}"
        channel = self.CHANS.get(channel, channel)
        refs = []
        for header in mail['references']:
            refs += header.split()
        for ref in refs:
            url = self.URL.format(**mail) + quote(f"/{ref}")
            print(f"GET {url}/raw")
            with urlopen(f"{url}/raw") as u:
                msg = mailbox.Message(u.read())
            subject = re.sub(r"\s+", " ", decode_header(msg["subject"]))
            if not re.match(r"^\[(RFC )?PATCH", subject):
                continue
            for name, addr in email.utils.getaddresses([decode_header(msg["from"])]):
                if name:
                    submitter = name
                else:
                    submitter = addr
                msg = f"{bold(mircColor('applied', 'green'))} {bold(subject)}"
                msg += f" from {italic(submitter)}: {underline(url)}"
                self.plugin.announce(channel, msg)
                return

    def doPost(self, handler, path, form=None):
        if hasattr(form, "decode"):
            form = form.decode("utf-8")
        print(f"POST {path} {form}")
        try:
            body = json.loads(form)
            hook = body["data"]["webhook"]
            if hook["event"] == "PATCHSET_RECEIVED":
                self.announce_patch(hook["patchset"])
                handler.send_response(200)
                handler.end_headers()
                handler.wfile.write(b"")
                return

            if hook["event"] == "EMAIL_RECEIVED":
                if hook["email"]["patchset_update"] == ["APPLIED"]:
                    self.announce_apply(hook["email"])
                handler.send_response(200)
                handler.end_headers()
                handler.wfile.write(b"")
                return

            raise ValueError(f"unsupported webhook: {hook}")

        except Exception as e:
            traceback.print_exception(e)
            handler.send_response(400)
            handler.end_headers()
            handler.wfile.write(b"Bad request\n")

    def log_message(self, format, *args):
        pass


Class = Sourcehut



#!/usr/bin/env python3
# SPDX-License-Identifier: MIT
# Copyright (c) 2023 Bence Ferdinandy <bence@ferdinandy.com>

"""
Create graphs about development statistics of releases.
"""

from datetime import date
from subprocess import check_output

from matplotlib import pyplot as plt


def git(*args):
    return check_output(["git"] + list(args)).decode("utf-8").strip()


def stats():
    """
    Returns statistics from the git repo:

    tags: sorted list of minor version tags (assumes semver)
          The first element is the hash of the first commit, last element is HEAD.  All
          the other return values are one shorter as, there's no statistics returned for
          the first commit.
    counts: number of commits (between this and the previous release)
    dates: dates of the releases
    files: number of files changed (between this and the previous release)
    inserts: number of lines inserted (between this and the previous release)
    deletions: number of lines deleted (between this and the previous release)
    """

    tags = git("tag").split("\n")
    tags = [t for t in tags if t.split(".")[-1] == "0"]  # drop patch versions
    tags = sorted(tags, key=lambda x: [int(t) for t in x.split(".")])
    first_commit = git("rev-list", "--max-parents=0", "HEAD")
    tags = [first_commit] + tags + ["HEAD"]
    counts = []
    dates = []
    files = []
    inserts = []
    deletions = []
    for i, t in enumerate(tags[:-1]):
        counts.append(int(git("rev-list", f"{t}..{tags[i+1]}", "--count")))
        dates.append(
            date.fromisoformat(
                git("show", "-s", "--format=%cs", tags[i + 1]).split("\n")[-1]
            )
        )
        statline = git("diff", "--stat", t, tags[i + 1]).split("\n")[-1]
        fnum, _, _, ins, _, dels, _ = statline.split()
        files.append(int(fnum))
        inserts.append(int(ins))
        deletions.append(int(dels))
    return tags, counts, dates, files, inserts, deletions


def main(output):
    tags, counts, dates, files, inserts, deletions = stats()

    fig, (ax1, ax2) = plt.subplots(2, figsize=(8, 11))
    fig.suptitle("aerc release statistics", fontweight="bold")
    # commit counts subplot
    ax1.plot(dates, counts, "o-")

    # alternate placement of text above and below for readability
    text_y = []
    for i, t in enumerate(tags[1:]):
        downpad = 25 if len(t) == 5 else 30
        p = counts[i] + (-1) ** (i + 1) * 10 - (i + 1) % 2 * downpad
        if p < 5:
            p = counts[i] + 10
        text_y.append(p)
    for i, t in enumerate(tags[1:]):
        ax1.text(
            dates[i],
            text_y[i],
            t,
            horizontalalignment="center",
            rotation="vertical",
        )
    ax1.set_ylabel("# of commits")
    ax1.set_ylim(bottom=0)
    ax1.set_title("commits per release")

    # lines added/deleted subplot
    #
    ax2.plot(dates, inserts, "o-", label="insertions(+)", color="green")
    ax2.plot(dates, deletions, "o-.", label="deletions(-)", color="red")
    ax2.set_ylabel("# of lines")
    ax2.legend(loc="upper left")
    ax2.set_ylim(top=max(max(inserts), max(deletions)) + 2000)
    for i, t in enumerate(tags[1:]):
        ax2.text(
            dates[i],
            max(inserts[i], deletions[i]) + 500,
            t,
            horizontalalignment="center",
            rotation="vertical",
        )
    ax2.set_xlabel("date")
    ax2.set_title("insertion/deletions per release")
    plt.tight_layout()
    plt.savefig(output, dpi=300)


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "-o",
        "--output",
        default="aerc-release-stats.png",
        help="""
        Path to output image (defaults to 'aerc-release-stats.png',
        respects file extensions via matplotlib)
        """,
    )
    args = parser.parse_args()
    main(args.output)



"""
Sourcehut: Supybot plugin to receive Sourcehut webhooks
"""

import sys
import supybot

__version__ = "0.1"
__author__ = supybot.authors.unknown
__contributors__ = {}
__url__ = ''

from . import config
from . import plugin
from importlib import reload

reload(config)
reload(plugin)

Class = plugin.Class
configure = config.configure



#!/usr/bin/env python3
# SPDX-License-Identifier: MIT
# Copyright (c) 2024 Robin Jarry

import argparse
import re
import subprocess

DEP_CHANGE_RE = re.compile(
    r"""
    ^
    (?P<diff>[\+\-])\s*
    (?P<name>\S+)\s*
    (?P<version>v\S+)\s*
    (?://\s*indirect)?
    $
    """,
    re.VERBOSE,
)
REPLACE_RE = re.compile(
    r"""
    ^
    (?P<diff>[\+\-])\s*
    replace
    (?P<name>\S+)\s*
    =>\s*
    (?P<replacement>\S+)\s*
    (?P<version>v\S+)\s*
    $
    """,
    re.VERBOSE,
)


def main():
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument(
        "git_range",
        metavar="GIT_RANGE",
        help="The git revision range (see gitrevisions(7)).",
    )
    args = parser.parse_args()

    old_deps = {}
    new_deps = {}

    with subprocess.Popen(
        ["git", "diff", "-U0", "--ignore-all-space", args.git_range, "--", "go.mod"],
        stdout=subprocess.PIPE,
        encoding="utf-8",
    ) as proc:
        for line in proc.stdout:
            match = DEP_CHANGE_RE.match(line.strip())
            if not match:
                match = REPLACE_RE.match(line.strip())
                if not match:
                    continue
                diff, name, replacement, version = match.groups()
                if diff == "+":
                    new_deps[replacement] = version
                    del new_deps[name]
                continue
            diff, name, version = match.groups()
            if diff == "+":
                new_deps[name] = version
            else:
                old_deps[name] = version

    once = False
    added = new_deps.keys() - old_deps.keys()
    if added:
        print("## New")
        print()
        for a in sorted(added):
            print("+", a, new_deps[a])
        once = True

    updated = old_deps.keys() & new_deps.keys()
    if updated:
        if once:
            print()
        print("## Updated")
        print()
        for u in sorted(updated):
            print("*", u, old_deps[u], "=>", new_deps[u])
        once = True

    removed = old_deps.keys() - new_deps.keys()
    if removed:
        if once:
            print()
        print("## Removed")
        print()
        for r in sorted(removed):
            print("-", r)
        once = True

    if not once:
        print("none")


if __name__ == "__main__":
    main()
